/*-----------------------------------------------------------------------------
 *  Copyright (c) 1998      TRIUMF Data Acquistion Group
 *  Please leave this header in any reproduction of that distribution
 * 
 *  TRIUMF Data Acquisition Group, 4004 Wesbrook Mall, Vancouver, B.C. V6T 2A3
 *  Email: online@triumf.ca           Tel: (604) 222-1047  Fax: (604) 222-1074
 *         amaudruz@triumf.ca
 * -----------------------------------------------------------------------------
 *  
 *  Description	: CAMAC interface for CES8210 CAMAC controller using
 *		  Midas Camac Standard calls.
 *  Requires 	: a VME driver:
 *                              VxVME.c-> VxWorks/PPC
 *                              btxxx  -> Linux,NT/bit3
 *  Application :
 *  Author      : Pierre-Andre Amaudruz Data Acquisition Group
 *  $Log: ces8210.c,v $
 *  Revision 1.2  2001/06/18 11:11:54  midas
 *  Major modifications (SR) to work with Bit3 MOdel 617 interface
 *
 *---------------------------------------------------------------------------*/

#ifndef INLINE 
#if defined( _MSC_VER )
#define INLINE __inline
#elif defined(__GNUC__)
#define INLINE __inline__
#else 
#define INLINE
#endif
#endif

#include <stdio.h>
#include <string.h>

#include "mcstd.h"             /* Provides CAMAC access */
#include "mvmestd.h"           /* using the generic Midas VME interface */

/* VME Base address for each crate */
typedef struct {
    int      baseCrate;        /* translated VME crate address */
} CBD_DESC_T;

CBD_DESC_T cbd[8];             /* crate 0 ==>system CC mapping */

/* VME CAMAC CBD8210 base address */
#define CBD8210_BASE   0x800000  /* camac base address */
#define CBDWL_D16      0x000002
#define CBDWL_D24      0x000000

/* VME handle */
int vh;

/*****************************************************************\
*                                                                 *
* Input functions                                                 *
*                                                                 *
\*****************************************************************/

INLINE void cam16i(const int c, const int n, const int a, const int f, 
                   WORD *d)
{
  int ext;
  
  ext = (cbd[c].baseCrate + CBDWL_D16 + (n<<11) + (a<<7) + (f<<2));
  *d = *((unsigned short *)ext);
}

/*---------------------------------------------------------------*/

INLINE void cam24i(const int c, const int n, const int a, const int f, 
                   DWORD *d)
{
int ext;
short int dh, dl;

  ext = (cbd[c].baseCrate + CBDWL_D24 + (n<<11) + (a<<7) + (f<<2));
  dh = *((unsigned short *)ext);
  ext += 2;
  dl = *((unsigned short *)ext);

  *d = ((dh<<16) | dl) & 0xffffff;
}

/*---------------------------------------------------------------*/

INLINE void cam16i_q(const int c, const int n, const int a, const int f,
                     WORD *d, int *x, int *q)
{
short int csr;

  cam16i(c,n,a,f, d);
  cami(0,29,0,0,&csr);
  *x = ((csr & 0x4000) != 0) ? 1 : 0;
  *q = ((csr & 0x8000) != 0) ? 1 : 0;
}

/*---------------------------------------------------------------*/

INLINE void cam24i_q(const int c, const int n, const int a, const int f,
                     DWORD *d, int *x, int *q)
{
short int csr;

  cam24i(c,n,a,f, d);
  cami(0,29,0,0,&csr);
  *x = ((csr & 0x4000) != 0) ? 1 : 0;
  *q = ((csr & 0x8000) != 0) ? 1 : 0;
}

/*---------------------------------------------------------------*/

INLINE void cam16i_r(const int c, const int n, const int a, const int f,
                     WORD **d, const int r)
{
int i;

  for (i=0 ; i<r ; i++)
    cam16i(c,n,a,f, (*d)++);
}

/*---------------------------------------------------------------*/

INLINE void cam24i_r(const int c, const int n, const int a, const int f,
                     DWORD **d, const int r)
{
  int i;

  for (i=0 ; i<r ; i++)
    cam24i(c,n,a,f, (*d)++);
}

/*---------------------------------------------------------------*/

INLINE void cam16i_rq(const int c, const int n, const int a, const int f,
                      WORD **d, const int r)
{
short int dtemp;
short int csr, i;

  for (i=0 ; i<r ; i++)
    {
    cam16i(c,n,a,f,&dtemp);
    cami(0,29,0,0,&csr);
    if ((csr & 0x8000) != 0)
      *((*d)++) = dtemp;
    else
      break;
    }
}

/*---------------------------------------------------------------*/

INLINE void cam24i_rq(const int c, const int n, const int a, const int f,
                      DWORD **d, const int r)
{
DWORD i, dtemp;
short int csr;

  for (i=0 ; i<(DWORD)r ; i++)
    {
    cam24i(c,n,a,f,&dtemp);
    cami(0,29,0,0,&csr);
    if ((csr & 0x8000) != 0)
      *((*d)++) = dtemp;
    else
      break;
    }
}

/*---------------------------------------------------------------*/

INLINE void cam16i_sa(const int c, const int n, const int a, const int f,
                      WORD **d, const int r)
{
int aa;

  for (aa=a ; aa<a+r ; aa++)
    cam16i(c,n,aa,f,*d++);
}

/*--input--------------------------------------------------------*/

INLINE void cam24i_sa(const int c, const int n, const int a, const int f,
                      unsigned long **d, const int r)
{
int aa;

  for (aa=a ; aa<a+r ; aa++)
    cam24i(c,n,aa,f,*d++);
}

/*---------------------------------------------------------------*/

INLINE void cam16i_sn(const int c, const int n, const int a, const int f,
                      WORD **d, const int r)
{
int nn;

  for (nn=n ; nn<n+r ; nn++)
    cam16i(c,nn,a,f,*d++);
}

/*---------------------------------------------------------------*/

INLINE void cam24i_sn(const int c, const int n, const int a, const int f,
                      DWORD **d, const int r)
{
int nn;

  for (nn=n ; nn<n+r ; nn++)
    cam24i(c,nn,a,f,*d++);
}

/*---------------------------------------------------------------*/

INLINE void cami(const int c, const int n, const int a, const int f
                 , WORD *d)
{
  cam16i(c,n,a,f, d);
}

/*****************************************************************\
*                                                                 *
* Output functions                                                *
*                                                                 *
\*****************************************************************/

INLINE void cam16o(const int c, const int n, const int a, const int f
                   , WORD d)
{
  int ext;
  
  ext = (cbd[c].baseCrate + CBDWL_D16 + (n<<11) + (a<<7) + (f<<2));
  *((unsigned short *)ext) = d;
}

/*---------------------------------------------------------------*/

INLINE void cam24o(const int c, const int n, const int a, const int f, 
                   DWORD d)
{
int ext;
unsigned short dtmp;

  ext = (cbd[c].baseCrate + CBDWL_D24 + (n<<11) + (a<<7) + (f<<2));
  dtmp = (unsigned short) ((d>>16) & 0xff);
  *((unsigned short *)ext) = dtmp;
  dtmp = (unsigned short) (d & 0xffff);
  ext += 2;
  *((unsigned short *)ext) = dtmp;
}

/*---------------------------------------------------------------*/

INLINE void cam16o_q(const int c, const int n, const int a, const int f,
                     WORD d, int *x, int *q)
{
short int csr;
  
  cam16o (c,n,a,f,d);
  cami(0,29,0,0,&csr);
  *x = ((csr & 0x4000) != 0) ? 1 : 0;
  *q = ((csr & 0x8000) != 0) ? 1 : 0;
}

/*---------------------------------------------------------------*/

INLINE void cam24o_q(const int c, const int n, const int a, const int f, 
                     DWORD d, int *x, int *q)
{
short int csr;

  cam24o (c,n,a,f,d);
  cami(0,29,0,0,&csr);
  *x = ((csr & 0x4000) != 0) ? 1 : 0;
  *q = ((csr & 0x8000) != 0) ? 1 : 0;
}

/*---------------------------------------------------------------*/

INLINE void camo(const int c, const int n, const int a, const int f
                 , WORD d)
{
  cam16o (c,n,a,f,d);
}

/*****************************************************************\
*                                                                 *
* Command functions                                               *
*                                                                 *
\*****************************************************************/

INLINE int camc_chk(const int c)
{
  return 1;
}

/*---------------------------------------------------------------*/

INLINE void camc(const int c, const int n, const int a, const int f)
{
short int dtmp;

  cami(c,n,a,f,&dtmp);
}

/*---------------------------------------------------------------*/

INLINE void camc_q(const int c, const int n, const int a, const int f, int *q)
{
short int csr;
  
  camc (c,n,a,f);
  cami(0,29,0,0,&csr);
  *q = ((csr & 0x8000) != 0) ? 1 : 0;
}

/*---------------------------------------------------------------*/

INLINE void camc_sa(const int c, const int n, const int a, const int f,
                    const int r)
{
int aa;

  for (aa=a ; aa<a+r ; aa++)
    camc(c,n,aa,f);
}

/*---------------------------------------------------------------*/

INLINE void camc_sn(const int c, const int n, const int a, const int f,
                    const int r)
{
int nn;

  for (nn=n ; nn<n+r ; nn++)
    camc(c,nn,a,f);
}

/*****************************************************************\
*                                                                 *
* General functions                                               *
*                                                                 *
\*****************************************************************/

INLINE int cam_init(void)
{
int branch=0, crate, vmecrate, err, am;

  /* Open in A24D16 */
  vh = vme_open(0, VME_A24D16);
  if (vh >= 0) 
    {
    am = VME_AMOD_A24_ND;
    vme_ioctl(vh, IOCTL_AMOD_SET, &am);
    vme_ioctl(vh, IOCTL_AMOD_GET, &am);
    
    /* mapping all crates */
    for (crate=0; crate<8; crate++) 
      {
      vmecrate = (CBD8210_BASE | branch << 19 | crate << 16);
      vme_mmap(vh, (void **) &(cbd[crate].baseCrate), vmecrate, 0x10000);
      /*
      printf("am 0x%x - baseCrate[%i] = 0x%x <- 0x%x\n", am, crate, 
              cbd[crate].baseCrate, vmecrate);
      */
      }
    }
  else 
    {
    vme_ioctl(vh, IOCTL_MAX_DEV_GET, &err);
    printf("cam_init: No more device space (>%d)\n", err);
    return 0;
    }

  return SUCCESS;
}

/*---------------------------------------------------------------*/

INLINE void cam_exit(void)
{
int crate;

  for (crate=0 ; crate<8 ; crate++)
    vme_unmap(vh, &cbd[crate].baseCrate, 0x10000);
  vme_close(vh);
}

/*---------------------------------------------------------------*/

INLINE void cam_inhibit_set(const int c)
{
  camc(c,30,9,26);
}

/*---------------------------------------------------------------*/

INLINE void cam_inhibit_clear(const int c)
{
  camc(c,30,9,24);
}

/*---------------------------------------------------------------*/

INLINE void cam_crate_clear(const int c)
{
  camc(c,28,9,26);
}

/*---------------------------------------------------------------*/

INLINE void cam_crate_zinit(const int c)
{
  camc(c,28,8,26);
}

/*---------------------------------------------------------------*/

INLINE void cam_lam_enable(const int c, const int n)
{
  camc(c,n,0,26);
}

/*---------------------------------------------------------------*/

INLINE void cam_lam_disable(const int c, const int n)
{
  camc(c,n,0,24);
}

/*---------------------------------------------------------------*/

INLINE void cam_lam_read(const int c, DWORD *lam)
{
  cam24i(0,29,0,10,lam);
}

/*---------------------------------------------------------------*/

INLINE void cam_lam_clear(const int c, const int n)
{
  /* Depend on the hardware LAM implementation
     as this cmd should talk to the controller
     but can include the LAM source module LAM clear
     camc (c,n,0,10); */
}

/*****************************************************************\
*                                                                 *
* Interrupt functions                                             *
*                                                                 *
\*****************************************************************/

INLINE void cam_interrupt_enable(void)
{
  printf("cam_interrupt_enable - Not yet implemented\n");
}

/*---------------------------------------------------------------*/

INLINE void cam_interrupt_disable(void)
{
  printf("cam_interrupt_disable - Not yet implemented\n");
}

/*---------------------------------------------------------------*/

INLINE void cam_interrupt_attach(void (*isr)(void))
{
  printf("cam_interrupt_attach - Not yet implemented\n");
}

/*---------------------------------------------------------------*/

INLINE void cam_interrupt_detach(void)
{
  printf("cam_interrupt_detach - Not yet implemented\n");
}

/*---------------------------------------------------------------*/

INLINE void cam_glint_enable(void)
{
  printf("cam_glint_enable - Not yet implemented\n");
}

/*---------------------------------------------------------------*/

INLINE void cam_glint_disable(void)
{
  printf("cam_glint_disable - Not yet implemented\n");
}

/*---------------------------------------------------------------*/

INLINE void cam_glint_attach(int lam, void (*isr)(void))
{
  printf("cam_glint_attach - Not yet implemented\n");
}

